@@ -25,7 +25,7 @@ try:
 except PackageNotFoundError:
     __version__ = '0.0.0'
 
-def check_latest_version():
+def check_latest_version(verbose=False):
     """ check for later versions on pypi """
     def echo(msg):
         if not os.getenv('OUTDATED_IGNORE'):
@@ -35,6 +35,9 @@ def check_latest_version():
         is_outdated, latest_version = check_outdated(PACKAGE_NAME, __version__)
         if is_outdated:
             echo(f'Your local version ({__version__}) is out of date! Latest is {latest_version}!')
+        elif verbose:
+            echo(f'You are running version {__version__}, latest is {latest_version}', file=sys.stderr)
+
     except ValueError:
         # this happens when your local version is ahead of the pypi version,
         # which happens only in development
@@ -133,8 +136,7 @@ def main():
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
@@ -143,7 +145,7 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     planfile, auto_approve, working_dir, terragrunt_args):
     """ [default] Executes a terragrunt command on a single project """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.run(
@@ -185,7 +187,8 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     help='Updates the source (and/or provider if applicable)',
     show_default=True
     )
-@click.option('--exclude-external-dependencies', '-x', is_flag=True, default=False,
+@click.option('--exclude-external-dependencies/--include-external-dependencies', '-x/-i',
+    is_flag=True, default=True,
     help='Whether or not external dependencies must be ignored',
     show_default=True
     )
@@ -201,16 +204,13 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
-@click.option(
-    '--start-at-step', '-S', type=float, default=1.0,
+@click.option('--start-at-step', '-S', type=float, default=1.0,
     help='When running in step-by-step mode, start processing at the given step number',
     )
-@click.option(
-    '--limit-parallelism', '-l', type=int, default=None,
+@click.option('--limit-parallelism', '-l', type=int, default=None,
     help='Limit the parallelism to the given number, unlimitted if omitted',
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
@@ -220,7 +220,7 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_
     limit_parallelism, terragrunt_args):
     """ Executes a terragrunt command across multiple projects """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.run_all(
@@ -245,12 +245,10 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_
         ignore_unknown_options=True,
     ),
 )
-@click.option(
-    '--address', '-a', required=True,
+@click.option('--address', '-a', required=True,
     help='Terraform resource address',
     )
-@click.option(
-    '--id', '-i', required=True,
+@click.option('--id', '-i', required=True,
     help='Id of the resource to be imported',
     )
 @click.option('--verbose', '-v', is_flag=True, default=False,
@@ -261,8 +259,7 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_
     help='dry-run mode, no real actions are executed',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.option('--no-lock', '-n', is_flag=True, default=False,
@@ -274,7 +271,7 @@ def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_
 def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_args):
     """ Executes the terragrunt/terraform import command """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.run_import(
@@ -300,16 +297,15 @@ def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_a
     help='dry-run mode, no real actions are executed (only in combination with step-by-step mode)',
     show_default=True
     )
-@click.option('--exclude-external-dependencies', '-x', is_flag=True, default=False,
+@click.option('--exclude-external-dependencies/--include-external-dependencies', '-x/-i',
+    is_flag=True, default=True,
     help='Whether or not external dependencies must be ignored',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
-@click.option(
-    '--start-at-step', '-S', type=float, default=1.0,
+@click.option('--start-at-step', '-S', type=float, default=1.0,
     help='When running in step-by-step mode, start processing at the given step number',
     )
 @click.option('--out', '-o', is_flag=True, default=False,
@@ -322,7 +318,7 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir,
     start_at_step, out, terragrunt_args):
     """ Analyzes the plan files """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.analyze(
@@ -335,31 +331,124 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir,
     )
 
 @main.command(
-    name="deploy",
+    name="lock",
     context_settings=dict(
         ignore_unknown_options=True,
     ),
 )
-@click.option(
-    '--source-domain', '-S', default="",
+@click.option('--domain', '-d', required=True,
+    help='Domain that needs to be locked',
+    )
+@click.option('--stage', '-s', required=True,
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Stage that needs to be unlocked.',
+    )
+@click.option('--module', '-m', required=True,
+    help='Name of the lockiung module (or set LOCK_MODULE environment variable)',
+    envvar='LOCK_MODULE'
+    )
+@click.option('--auto-approve', '-a', is_flag=True, default=False,
+    help='Do not ask for confirmation before applying planned changes (where applicable)',
+    show_default=True
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def lock(domain, stage, module, auto_approve, verbose, dry_run, working_dir):
+    """ Lock a particular domain/stage """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.set_lock(
+        domain=domain,
+        stage=stage,
+        module=module,
+        lock_status="lock",
+        auto_approve=auto_approve,
+        dry_run=dry_run,
+        working_dir=working_dir,
+    )
+
+@main.command(
+    name="unlock",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--domain', '-d', required=True,
+    help='Domain that needs to be locked',
+    )
+@click.option('--stage', '-s', required=True,
+    type=click.Choice(STAGES, case_sensitive=True),
+    help='Stage that needs to be unlocked.',
+    )
+@click.option('--module', '-m', required=True,
+    help='Name of the lockiung module (or set LOCK_MODULE environment variable)',
+    envvar='LOCK_MODULE'
+    )
+@click.option('--auto-approve', '-a', is_flag=True, default=False,
+    help='Do not ask for confirmation before applying planned changes (where applicable)',
+    show_default=True
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--dry-run', '-D', is_flag=True, default=False,
+    help='dry-run mode, no real actions are executed',
+    show_default=True
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def unlock(domain, stage, module, auto_approve, verbose, dry_run, working_dir):
+    """ Unlock a particular domain/stage """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.set_lock(
+        domain=domain,
+        stage=stage,
+        module=module,
+        lock_status="unlock",
+        auto_approve=auto_approve,
+        dry_run=dry_run,
+        working_dir=working_dir,
+    )
+
+@main.command(
+    name="sync",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--source-domain', '-S', default="",
     help='Source domain of config files, when omitted the DLZ where you run this command is assumed.',
     )
-@click.option(
-    '--target-domain', '-T', default="",
+@click.option('--target-domain', '-T', default="",
     help='Target domain where config files will be copied to, when omitted the DLZ where you run this command is assumed.',
     )
-@click.option(
-    '--source-stage', '-s', required=True,
+@click.option('--source-stage', '-s', required=True,
     type=click.Choice(STAGES, case_sensitive=True),
     help='Source stage of config files',
     )
-@click.option(
-    '--target-stage', '-t',
+@click.option('--target-stage', '-t',
     type=click.Choice(STAGES, case_sensitive=True),
     help='Target of config files',
     )
-@click.option(
-    '--module', '-m', default="",
+@click.option('--module', '-m', default="",
     help='Name of the module, if omitted all modules will be copied.',
     )
 @click.option('--clean-up', '-c', is_flag=True, default=False,
@@ -382,21 +471,20 @@ def analyze(verbose, dry_run, exclude_external_dependencies, working_dir,
     help='dry-run mode, no real actions are executed',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.version_option(version=__version__)
-def deploy(
+def sync(
     source_domain, source_stage, target_domain, target_stage, module, auto_approve,
     verbose, dry_run, clean_up, include_lock_file, working_dir
     ):
     """ Deploys the terragrunt config files from one stage to another (and possibly to a different domain) """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
-    tgwrap.deploy(
+    tgwrap.sync(
         source_domain=source_domain,
         source_stage=source_stage,
         target_domain=target_domain,
@@ -410,27 +498,20 @@ def deploy(
     )
 
 @main.command(
-    name="git-deploy",
+    name="sync-dir",
     context_settings=dict(
         ignore_unknown_options=True,
     ),
 )
-@click.option(
-    '--git-repo', '-g', envvar='TERRAGRUNT_GIT_REPO',
-    help='URL of the git repository that contains the terragrunt files',
+@click.option('--source-directory', '-s', required=True,
+    help='Directory where source config files reside.',
     )
-@click.option(
-    '--source-directory', '-S',
-    help='Directory (in git repo) where terragrunt files are located',
+@click.option('--target-directory', '-t', required=True,
+    help='Directory where config files will be synced to.',
     )
-@click.option(
-    '--target-domain', '-T', default="",
-    help='Target domain where config files will be copied to, when omitted the project where you run this command is assumed.',
-    )
-@click.option(
-    '--target-stage', '-t',
-    type=click.Choice(STAGES, case_sensitive=True),
-    help='Target of config files',
+@click.option('--clean-up', '-c', is_flag=True, default=False,
+    help='Clean up files on target side that do not exist as source',
+    show_default=True
     )
 @click.option('--include-lock-file', '-i', is_flag=True, default=False,
     help='Include the terraform lock file',
@@ -448,28 +529,78 @@ def deploy(
     help='dry-run mode, no real actions are executed',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.version_option(version=__version__)
-def git_deploy(
-    git_repo, source_directory, target_domain, target_stage,
-    auto_approve, verbose, dry_run, include_lock_file, working_dir
+def sync_dir(
+    source_directory, target_directory, auto_approve,
+    verbose, dry_run, clean_up, include_lock_file, working_dir
+    ):
+    """ Syncs the terragrunt config files from one directory to anothery """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.sync_dir(
+        source_directory=source_directory,
+        target_directory=target_directory,
+        auto_approve=auto_approve,
+        dry_run=dry_run,
+        clean_up=clean_up,
+        include_lock_file=include_lock_file,
+        working_dir=working_dir,
+    )
+
+@main.command(
+    name="deploy",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--manifest-file', '-m',
+    help='Manifest file describing the deployment options',
+    required=True, default="manifest.yaml", show_default=True,
+    )
+@click.option('--version-tag', '-V',
+    help="Version tag, use 'latest' to, well, get the latest version.",
+    required=True, default='latest', show_default=True,
+    )
+@click.option('--target-stage', '-t',
+    help='Stage to deploy to',
+    type=click.Choice(STAGES, case_sensitive=True), required=True,
+    )
+@click.option('--auto-approve', '-a',
+    help='Do not ask for confirmation before applying planned changes (where applicable)',
+    is_flag=True, default=False, show_default=True,
+    )
+@click.option('--verbose', '-v',
+    help='Verbose printing',
+    is_flag=True, default=False, show_default=True,
+    )
+@click.option('--dry-run', '-D',
+    help='dry-run mode, no real actions are executed',
+    is_flag=True, default=False, show_default=True,
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.version_option(version=__version__)
+def deploy(
+    manifest_file, version_tag, target_stage,
+    auto_approve, verbose, dry_run, working_dir
     ):
     """ [experimental] Deploys the terragrunt config files from a git repository """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
-    tgwrap.git_deploy(
-        git_repo=git_repo,
-        source_directory=source_directory,
-        target_domain=target_domain,
+    tgwrap.deploy(
+        manifest_file=manifest_file,
+        version_tag=version_tag,
         target_stage=target_stage,
         auto_approve=auto_approve,
         dry_run=dry_run,
-        include_lock_file=include_lock_file,
         working_dir=working_dir,
     )
 
@@ -487,8 +618,7 @@ def git_deploy(
     help='Whether or not external dependencies must be ignored',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
@@ -496,7 +626,7 @@ def git_deploy(
 def show_graph(verbose, exclude_external_dependencies, working_dir, terragrunt_args):
     """ Shows the dependencies of a project """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.show_graph(
@@ -515,15 +645,14 @@ def show_graph(verbose, exclude_external_dependencies, working_dir, terragrunt_a
     help='Verbose printing',
     show_default=True
     )
-@click.option(
-    '--working-dir', '-w', default=None,
+@click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.version_option(version=__version__)
 def clean(verbose, working_dir):
     """ Clean the temporary files of a terragrunt/terraform project """
 
-    check_latest_version()
+    check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.clean(
