@@ -20,6 +20,7 @@ import shutil
 import re
 import tempfile
 import json
+import yaml
 import click
 import hcl
 import networkx as nx
@@ -52,6 +53,10 @@ class TgWrap():
         """ Checks if a program is installed on the system """
         return shutil.which(program) is not None
 
+    def load_yaml_file(self, filepath):
+        with open(filepath, 'r') as file:
+            return yaml.safe_load(file)
+
     def _construct_command(self, command, debug, exclude_external_dependencies,
         non_interactive=True, no_auto_approve=True, no_lock=True, update=False,
         planfile=None, working_dir=None, limit_parallelism=None, terragrunt_args=()):
@@ -303,7 +308,7 @@ class TgWrap():
                 nbr_of_modules = len(group)
                 for idx2, module in enumerate(group):
                     module_nbr=idx2+1
-                    progress = f'module #{module_nbr}/{nbr_of_modules} of group #{group_nbr}/{nbr_of_groups}'
+                    progress = f'module {module_nbr} (of {nbr_of_modules}) of group {group_nbr} (of {nbr_of_groups})'
 
                     step_nbr = group_nbr + module_nbr/10
                     if step_nbr < start_at_step:
@@ -320,6 +325,10 @@ class TgWrap():
                     )
 
                     if stop_processing:
+                        self.printer.normal(f"Processing aborted at step {group_nbr}.{module_nbr}.")
+                        self.printer.normal(
+                            f"After you've fixed the problem, you can continue where you were left by adding '--start-at-step {group_nbr}.{module_nbr}'."
+                            )
                         break
 
             if stop_processing:
@@ -328,8 +337,8 @@ class TgWrap():
         self.printer.verbose(f'Executed {group_nbr} groups and {step_nbr} steps')
 
     def _run_sync(
-        self, source_path, target_path, source_stage, target_stage, auto_approve,
-        include_lock_file, dry_run, clean_up, git_source_path=None, source_domain=None,
+        self, source_path, target_path, auto_approve, include_lock_file, dry_run, clean_up,
+        source_stage=None, target_stage=None, source_domain=None,
         ):
         """ Run a sync copying files from a source to a target path """
 
@@ -371,7 +380,7 @@ class TgWrap():
             cmd = re.sub(' +', ' ', cmd)
 
             self.printer.header("Will be deploying:", print_line_before=True)
-            self.printer.normal(f"from: {git_source_path if git_source_path else source_path}")
+            self.printer.normal(f"from: {source_path}")
             self.printer.normal(f"to:   {target_path}")
             self.printer.verbose(f"Using command:\n$ {cmd}")
 
@@ -638,7 +647,28 @@ class TgWrap():
             self.printer.error("Terrasafe validation failed on one or more modules")
             sys.exit(1)
 
-    def deploy(
+    def set_lock(self, domain, stage, module, lock_status, auto_approve, dry_run, working_dir):
+        """ Set the lock status of a particular domain/stage """
+    
+        # do we have a working dir?
+        working_dir = working_dir if working_dir else os.getcwd()
+        module_path = os.path.join(working_dir, domain, stage, module)
+
+        command = "destroy" if lock_status == "unlock" else "apply" 
+
+        self.run(
+            command=command,
+            dry_run=dry_run,
+            auto_approve=auto_approve,
+            working_dir=module_path,
+            debug=False,
+            no_lock=False,
+            update=False,
+            planfile=None,
+            terragrunt_args=[],
+            )
+
+    def sync(
         self, source_stage, target_stage, source_domain, target_domain, module,
         auto_approve, dry_run, clean_up, include_lock_file, working_dir, 
         ):
@@ -674,37 +704,53 @@ class TgWrap():
             clean_up=clean_up,
         )
 
-    def git_deploy(
-        self, git_repo, source_directory, target_domain, target_stage,
-        auto_approve, dry_run, include_lock_file, working_dir, 
+    def sync_dir(
+        self, source_directory, target_directory,
+        auto_approve, dry_run, clean_up, include_lock_file, working_dir, 
+        ):
+        """ Syncs the terragrunt config files from one directory to anothery """
+    
+        # do we have a working dir?
+        working_dir = working_dir if working_dir else os.getcwd()
+        # the domains will be ignored when omitted as input
+        source_path = os.path.join(working_dir, source_directory, '')
+        target_path = os.path.join(working_dir, target_directory, '')
+
+        self._run_sync(
+            source_path=source_path,
+            target_path=target_path,
+            include_lock_file=include_lock_file,
+            auto_approve=auto_approve,
+            dry_run=dry_run,
+            clean_up=clean_up,
+        )
+
+    def deploy(
+        self, manifest_file, version_tag, target_stage,
+        auto_approve, dry_run, working_dir, 
         ):
         """ Deploys the terragrunt config files from a git repository """
 
         DEFAULT_VERSION='latest'
 
-        # first try to determine the desired version, this should be coded in
-        # an `version.hcl` file in the root of the target
-        version_file = os.path.join(os.getcwd(), target_domain, target_stage, "version.hcl")
-        version = DEFAULT_VERSION # default
-        if os.path.isfile(version_file):
-            self.printer.verbose(f'Try to get desired version info from {version_file}')
-            with open(version_file, 'r') as f:
-                version_info = hcl.load(f)
-
-            try:
-                version = version_info['locals'].get('platform_version')
-            except AttributeError as e:
-                click.warning(f'Could not determine platform version from file: {e}')
-        else:
-            self.printer.warning(f"Version file {version_file}'' not found, use '{DEFAULT_VERSION}' as version.")
-        
-        self.printer.verbose(f"Using version '{version}' as source")
-
         try:
             temp_dir = os.path.join(tempfile.mkdtemp(prefix='tgwrap-'), "tg-source")
 
+            # do we have a working dir? 
+            working_dir = working_dir if working_dir else os.getcwd()
+
+            manifest = self.load_yaml_file(os.path.join(working_dir, manifest_file))
+
+            source_dir = os.path.join(temp_dir, manifest['base_path'])
+            target_dir = os.path.join(working_dir, target_stage)
+
+            try:
+                os.mkdir(target_dir)
+            except FileExistsError:
+                pass
+
             # clone the repo
-            cmd = f"git clone {git_repo} {temp_dir}"
+            cmd = f"git clone {manifest['git_repository']} {temp_dir}"
             rc = subprocess.run(
                 shlex.split(cmd),
                 check=True,
@@ -714,8 +760,8 @@ class TgWrap():
             self.printer.verbose(rc)
 
             # now check out the specific version if we don't want latest
-            if version != DEFAULT_VERSION:
-                cmd = f"git checkout -b source {version}"
+            if version_tag.lower() != DEFAULT_VERSION:
+                cmd = f"git checkout -b source {version_tag}"
                 rc = subprocess.run(
                     shlex.split(cmd),
                     cwd=temp_dir,
@@ -725,46 +771,86 @@ class TgWrap():
                     )
                 self.printer.verbose(rc)
 
-            # do we have a working dir?
-            working_dir = working_dir if working_dir else os.getcwd()
+            # collect all the base paths of the substacks as you don't want
+            # to include them in regular syncs
+            substacks = []
+            for ss, substack in manifest['sub_stacks'].items():
+                # get the base directory of the sub stack so that we can ignore it when deploying the regular modules
+                substacks.append(substack['source_path'].split(os.path.sep)[0])
 
-            # run some sanity checks
-            reversed_path = list(reversed(source_directory.split('/')))
-            source_stage = reversed_path[0]
-            source_type = reversed_path[1]
+            deploy_actions = {}
+            for key, value in manifest['deploy'].items():
+                if target_stage not in value['applies_to_stages']:
+                    self.printer.verbose(f'Target stage {target_stage} not applicable for action {key}.')
+                else:
+                    self.printer.verbose(f'Found deployment step: {key}')
 
-            if target_stage == 'global' and source_stage != target_stage:
-                self.printer.error(f'You are trying to copy source stage {source_stage} to a global stage!')
-                self.printer.error('This is not correct, exit.')
-                sys.exit(1)
+                    source_path = os.path.join(source_dir, value['source_stage'])
+                    source_modules = [
+                        entry for entry in os.listdir(source_path) if os.path.isdir(os.path.join(source_path, entry))
+                        ]
+                    self.printer.verbose(f'Found modules: {source_modules}')
 
-            reversed_path = list(reversed(working_dir.split('/')))
-            target_type = reversed_path[0]
-            if target_type.upper() == 'DLZS' and not target_domain:
-                self.printer.warning(
-                    f'Detected that you are targetting a landing zone project but you provided no target domain!'
-                    )
+                    for module in source_modules:
+                        if module in value['exclude_modules'] or module in substacks:
+                            self.printer.verbose(f'Exclude module {module}')
+                        elif module in value['include_modules'] or len(value['include_modules']) == 0:
+                            deploy_actions[module] = {
+                                "source": os.path.join(source_path, module, ''),
+                                "target": os.path.join(target_dir, module, ''),
+                            }
 
-            # the source stage is either 'global' (if target == global) or 'dev'
-            source_stage = target_stage if target_stage == 'global' else 'dev'
-            # construct the paths
-            source_path = os.path.join(temp_dir, source_directory, '')
-            target_path = os.path.join(working_dir, target_domain, target_stage, '')
+                    for ss, substack in manifest['sub_stacks'].items():
+                        self.printer.verbose(f'Found substack : {ss}')
 
-            self._run_sync(
-                source_path=source_path,
-                git_source_path=f'{git_repo}//{source_directory}?ref={version}',
-                target_path=target_path,
-                source_stage=source_stage,
-                target_stage=target_stage,
-                include_lock_file=include_lock_file,
-                auto_approve=auto_approve,
-                dry_run=dry_run,
-                clean_up=False,
-            )
+                        source_path = os.path.join(
+                            source_dir, value['source_stage'], substack['source_path'], ''
+                            )
+                        target_path = os.path.join(
+                            target_dir, substack['target_path'], ''
+                            )
 
+                        if os.path.exists(source_path):
+                            deploy_actions[f'substack -> {substack["target_path"]}'] = {
+                                "source": source_path,
+                                "target": target_path,
+                            }
+                        else:
+                            self.printer.warning(f'Source path of substack does not exist: {source_path}')
+
+            self.printer.header('Modules to deploy:')
+            self.printer.normal(f'-> git repository: {manifest["git_repository"]}')
+            self.printer.normal(f'-> version tag: {version_tag}')
+            self.printer.normal('Modules:')
+            for key, value in deploy_actions.items():
+                self.printer.normal(f'--> {key}')
+
+            if not auto_approve:
+                response = input("\nDo you want to continue? (y/n) ")
+                if response.lower() != "y":
+                    sys.exit(1)
+
+            for key, value in deploy_actions.items():
+                self._run_sync(
+                    source_path=value['source'],
+                    target_path=value['target'],
+                    include_lock_file=True,
+                    auto_approve=True,
+                    dry_run=dry_run,
+                    clean_up=True,
+                )
+
+            with open(os.path.join(target_dir, 'version.hcl'), 'w') as f:
+                f.write(f"""
+locals {{
+    version_tag="{version_tag}"
+}}
+""")
         finally:
-            shutil.rmtree(temp_dir)
+            try:
+                shutil.rmtree(temp_dir)
+            except Exception:
+                pass
 
     def show_graph(self, exclude_external_dependencies, working_dir, terragrunt_args):
         """ Shows the dependencies of a project """
